'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.default = getCommand;

var _env = require('../utils/env');

var _env2 = _interopRequireDefault(_env);

var _options = require('../utils/options');

var _options2 = _interopRequireDefault(_options);

var _query = require('../utils/query');

var _query2 = _interopRequireDefault(_query);

var _constants = require('../utils/constants');

var _print = require('../utils/print');

var _helpers = require('../utils/helpers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } } /*
                                                                                                                                                                                                     * LiskHQ/lisky
                                                                                                                                                                                                     * Copyright © 2017 Lisk Foundation
                                                                                                                                                                                                     *
                                                                                                                                                                                                     * See the LICENSE file at the top-level directory of this distribution
                                                                                                                                                                                                     * for licensing information.
                                                                                                                                                                                                     *
                                                                                                                                                                                                     * Unless otherwise agreed in a custom licensing agreement with the Lisk Foundation,
                                                                                                                                                                                                     * no part of this software, including this file, may be copied, modified,
                                                                                                                                                                                                     * propagated, or distributed except according to the terms contained in the
                                                                                                                                                                                                     * LICENSE file.
                                                                                                                                                                                                     *
                                                                                                                                                                                                     * Removal or modification of this copyright notice is prohibited.
                                                                                                                                                                                                     *
                                                                                                                                                                                                     */


var handlers = {
	account: function account(_account) {
		return _query2.default.isAccountQuery(_account);
	},
	address: function address(_address) {
		return _query2.default.isAccountQuery(_address);
	},
	block: function block(_block) {
		return _query2.default.isBlockQuery(_block);
	},
	delegate: function delegate(_delegate) {
		return _query2.default.isDelegateQuery(_delegate);
	},
	transaction: function transaction(_transaction) {
		return _query2.default.isTransactionQuery(_transaction);
	}
};

var processResult = function processResult(useJsonOutput, vorpal, type, result) {
	var resultToPrint = result.error ? result : result[type];
	return (0, _print.printResult)(vorpal, { json: useJsonOutput })(resultToPrint);
};

var get = function get(vorpal) {
	return function (_ref) {
		var options = _ref.options,
		    type = _ref.type,
		    input = _ref.input;

		var useJsonOutput = (0, _helpers.shouldUseJsonOutput)(_env2.default, options);
		return _constants.COMMAND_TYPES.includes(type) ? handlers[type](input).then(processResult.bind(null, useJsonOutput, vorpal, (0, _helpers.deAlias)(type))) : Promise.resolve(vorpal.activeCommand.log('Unsupported type.'));
	};
};

function getCommand(vorpal) {
	var _vorpal$command$optio, _vorpal$command;

	(_vorpal$command$optio = (_vorpal$command = vorpal.command('get <type> <input>')).option.apply(_vorpal$command, _toConsumableArray(_options2.default.json))).option.apply(_vorpal$command$optio, _toConsumableArray(_options2.default.noJson)).description('Get information from <type> with parameter <input>.\n  Types available: account, address, block, delegate, transaction\n  Example: get delegate lightcurve\n  Example: get block 5510510593472232540').autocomplete(_constants.COMMAND_TYPES).action(get(vorpal));
}