'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _defaultConfig = require('../../defaultConfig.json');

var _defaultConfig2 = _interopRequireDefault(_defaultConfig);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var configDirName = '.lisky'; /*
                               * LiskHQ/lisky
                               * Copyright © 2017 Lisk Foundation
                               *
                               * See the LICENSE file at the top-level directory of this distribution
                               * for licensing information.
                               *
                               * Unless otherwise agreed in a custom licensing agreement with the Lisk Foundation,
                               * no part of this software, including this file, may be copied, modified,
                               * propagated, or distributed except according to the terms contained in the
                               * LICENSE file.
                               *
                               * Removal or modification of this copyright notice is prohibited.
                               *
                               */

var configFileName = 'config.json';
var homedir = _os2.default.homedir();
var configDirPath = homedir + '/' + configDirName;
var configFilePath = configDirPath + '/' + configFileName;

var attemptCallWithWarning = function attemptCallWithWarning(fn, path) {
	try {
		return fn();
	} catch (_) {
		var warning = 'WARNING: Could not write to `' + path + '`. Your configuration will not be persisted.';
		console.warn(warning);
		return null;
	}
};

var attemptCallWithError = function attemptCallWithError(fn, code, error) {
	try {
		return fn();
	} catch (_) {
		console.error(error);
		return process.exit(code);
	}
};

var attemptToCreateDir = function attemptToCreateDir(path) {
	var fn = _fsExtra2.default.mkdirSync.bind(null, path);
	return attemptCallWithWarning(fn, path);
};

var attemptToCreateFile = function attemptToCreateFile(path) {
	var fn = _fsExtra2.default.writeJsonSync.bind(null, path, _defaultConfig2.default, {
		spaces: '\t'
	});
	return attemptCallWithWarning(fn, path);
};

var checkReadAccess = function checkReadAccess(path) {
	var fn = _fsExtra2.default.accessSync.bind(null, path, _fsExtra2.default.constants.R_OK);
	var error = 'Could not read config file. Please check permissions for ' + path + ' or delete the file so we can create a new one from defaults.';
	return attemptCallWithError(fn, 1, error);
};

var attemptToReadJsonFile = function attemptToReadJsonFile(path) {
	var fn = _fsExtra2.default.readJsonSync.bind(null, path);
	var error = 'Config file is not valid JSON. Please check ' + path + ' or delete the file so we can create a new one from defaults.';
	return attemptCallWithError(fn, 2, error);
};

var getConfig = function getConfig() {
	if (!_fsExtra2.default.existsSync(configDirPath)) {
		attemptToCreateDir(configDirPath);
	}

	if (!_fsExtra2.default.existsSync(configFilePath)) {
		attemptToCreateFile(configFilePath);
		return _defaultConfig2.default;
	}

	checkReadAccess(configFilePath);

	return attemptToReadJsonFile(configFilePath);
};

exports.default = getConfig();