'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.getData = exports.getDataFromFile = exports.getFirstLineFromString = exports.getPassphrase = exports.getPassphraseFromSource = exports.getPassphraseFromFile = exports.getPassphraseFromEnvVariable = exports.getPassphraseFromPrompt = exports.createPromptOptions = exports.getStdIn = exports.splitSource = undefined;

var _readline = require('readline');

var _readline2 = _interopRequireDefault(_readline);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; } /*
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * LiskHQ/lisky
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * Copyright © 2017 Lisk Foundation
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * See the LICENSE file at the top-level directory of this distribution
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * for licensing information.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * Unless otherwise agreed in a custom licensing agreement with the Lisk Foundation,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * no part of this software, including this file, may be copied, modified,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * propagated, or distributed except according to the terms contained in the
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * LICENSE file.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * Removal or modification of this copyright notice is prohibited.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            */


var capitalise = function capitalise(text) {
	return '' + text.charAt(0).toUpperCase() + text.slice(1);
};

var getPassphraseVerificationFailError = function getPassphraseVerificationFailError(displayName) {
	return capitalise(displayName) + ' was not successfully repeated.';
};
var getPassphraseSourceTypeUnknownError = function getPassphraseSourceTypeUnknownError(displayName) {
	return capitalise(displayName) + ' was provided with an unknown source type. Must be one of `env`, `file`, or `stdin`. Leave blank for prompt.';
};
var getPassphraseEnvVariableNotSetError = function getPassphraseEnvVariableNotSetError(displayName) {
	return 'Environmental variable for ' + displayName + ' not set.';
};
var getFileDoesNotExistError = function getFileDoesNotExistError(path) {
	return 'File at ' + path + ' does not exist.';
};
var getFileUnreadableError = function getFileUnreadableError(path) {
	return 'File at ' + path + ' could not be read.';
};
var ERROR_DATA_MISSING = 'No data was provided.';
var ERROR_DATA_SOURCE_TYPE_UNKNOWN = 'Unknown data source type. Must be one of `file`, or `stdin`.';

var splitSource = exports.splitSource = function splitSource(source) {
	var delimiter = ':';
	var sourceParts = source.split(delimiter);
	return {
		sourceType: sourceParts[0],
		sourceIdentifier: sourceParts.slice(1).join(delimiter)
	};
};

var getStdIn = exports.getStdIn = function getStdIn() {
	var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
	    dataIsRequired = _ref.dataIsRequired,
	    passphraseIsRequired = _ref.passphraseIsRequired;

	return new Promise(function (resolve) {
		if (!dataIsRequired && !passphraseIsRequired) return resolve({});

		var lines = [];
		var rl = _readline2.default.createInterface({ input: process.stdin });

		var handleLine = function handleLine(line) {
			return dataIsRequired ? lines.push(line) : resolve({ passphrase: line }) || rl.close();
		};
		var handleClose = function handleClose() {
			var dataLines = passphraseIsRequired ? lines.slice(1) : lines;
			return resolve({
				data: dataLines.join('\n'),
				passphrase: passphraseIsRequired ? lines[0] : null
			});
		};

		return rl.on('line', handleLine).on('close', handleClose);
	});
};

var createPromptOptions = exports.createPromptOptions = function createPromptOptions(message) {
	return {
		type: 'password',
		name: 'passphrase',
		message: message
	};
};

var getPassphraseFromPrompt = exports.getPassphraseFromPrompt = function getPassphraseFromPrompt(vorpal, _ref2) {
	var displayName = _ref2.displayName,
	    shouldRepeat = _ref2.shouldRepeat;

	// IMPORTANT: prompt will exit if UI has no parent, but calling
	// ui.attach(vorpal) will start a prompt, which will complain when we call
	// vorpal.activeCommand.prompt(). Therefore set the parent directly.
	if (!vorpal.ui.parent) {
		// eslint-disable-next-line no-param-reassign
		vorpal.ui.parent = vorpal;
	}

	var handleSecondPassphrase = function handleSecondPassphrase(passphrase) {
		return vorpal.activeCommand.prompt(createPromptOptions('Please re-enter ' + displayName + ': ')).then(function (_ref3) {
			var passphraseRepeat = _ref3.passphrase;

			if (passphrase !== passphraseRepeat) {
				throw new Error(getPassphraseVerificationFailError(displayName));
			}
			return passphrase;
		});
	};

	var handleFirstPassphrase = function handleFirstPassphrase(_ref4) {
		var passphrase = _ref4.passphrase;
		return shouldRepeat ? handleSecondPassphrase(passphrase) : passphrase;
	};

	return vorpal.activeCommand.prompt(createPromptOptions('Please enter ' + displayName + ': ')).then(handleFirstPassphrase);
};

var getPassphraseFromEnvVariable = exports.getPassphraseFromEnvVariable = function () {
	var _ref5 = _asyncToGenerator(regeneratorRuntime.mark(function _callee(key, displayName) {
		var passphrase;
		return regeneratorRuntime.wrap(function _callee$(_context) {
			while (1) {
				switch (_context.prev = _context.next) {
					case 0:
						passphrase = process.env[key];

						if (passphrase) {
							_context.next = 3;
							break;
						}

						throw new Error(getPassphraseEnvVariableNotSetError(displayName));

					case 3:
						return _context.abrupt('return', passphrase);

					case 4:
					case 'end':
						return _context.stop();
				}
			}
		}, _callee, undefined);
	}));

	return function getPassphraseFromEnvVariable(_x2, _x3) {
		return _ref5.apply(this, arguments);
	};
}();

var getPassphraseFromFile = exports.getPassphraseFromFile = function getPassphraseFromFile(path) {
	return new Promise(function (resolve, reject) {
		var stream = _fsExtra2.default.createReadStream(path);
		var handleReadError = function handleReadError(error) {
			stream.close();
			var message = error.message;


			if (message.match(/ENOENT/)) {
				return reject(new Error(getFileDoesNotExistError(path)));
			}
			if (message.match(/EACCES/)) {
				return reject(new Error(getFileUnreadableError(path)));
			}

			return reject(error);
		};
		var handleLine = function handleLine(line) {
			stream.close();
			resolve(line);
		};

		stream.on('error', handleReadError);

		_readline2.default.createInterface({ input: stream }).on('error', handleReadError).on('line', handleLine);
	});
};

var getPassphraseFromSource = exports.getPassphraseFromSource = function () {
	var _ref7 = _asyncToGenerator(regeneratorRuntime.mark(function _callee2(source, _ref6) {
		var displayName = _ref6.displayName;

		var _splitSource, sourceType, sourceIdentifier;

		return regeneratorRuntime.wrap(function _callee2$(_context2) {
			while (1) {
				switch (_context2.prev = _context2.next) {
					case 0:
						_splitSource = splitSource(source), sourceType = _splitSource.sourceType, sourceIdentifier = _splitSource.sourceIdentifier;
						_context2.t0 = sourceType;
						_context2.next = _context2.t0 === 'env' ? 4 : _context2.t0 === 'file' ? 5 : _context2.t0 === 'pass' ? 6 : 7;
						break;

					case 4:
						return _context2.abrupt('return', getPassphraseFromEnvVariable(sourceIdentifier, displayName));

					case 5:
						return _context2.abrupt('return', getPassphraseFromFile(sourceIdentifier));

					case 6:
						return _context2.abrupt('return', sourceIdentifier);

					case 7:
						throw new Error(getPassphraseSourceTypeUnknownError(displayName));

					case 8:
					case 'end':
						return _context2.stop();
				}
			}
		}, _callee2, undefined);
	}));

	return function getPassphraseFromSource(_x4, _x5) {
		return _ref7.apply(this, arguments);
	};
}();

var getPassphrase = exports.getPassphrase = function () {
	var _ref8 = _asyncToGenerator(regeneratorRuntime.mark(function _callee3(vorpal, passphraseSource, passphrase, options) {
		var optionsWithDefaults;
		return regeneratorRuntime.wrap(function _callee3$(_context3) {
			while (1) {
				switch (_context3.prev = _context3.next) {
					case 0:
						optionsWithDefaults = Object.assign({ displayName: 'your secret passphrase' }, options);

						if (!passphrase) {
							_context3.next = 3;
							break;
						}

						return _context3.abrupt('return', passphrase);

					case 3:
						if (passphraseSource) {
							_context3.next = 5;
							break;
						}

						return _context3.abrupt('return', getPassphraseFromPrompt(vorpal, optionsWithDefaults));

					case 5:
						return _context3.abrupt('return', getPassphraseFromSource(passphraseSource, optionsWithDefaults));

					case 6:
					case 'end':
						return _context3.stop();
				}
			}
		}, _callee3, undefined);
	}));

	return function getPassphrase(_x6, _x7, _x8, _x9) {
		return _ref8.apply(this, arguments);
	};
}();

var getFirstLineFromString = exports.getFirstLineFromString = function getFirstLineFromString(multilineString) {
	return typeof multilineString === 'string' ? multilineString.split(/[\r\n]+/)[0] : null;
};

var getDataFromFile = exports.getDataFromFile = function () {
	var _ref9 = _asyncToGenerator(regeneratorRuntime.mark(function _callee4(path) {
		return regeneratorRuntime.wrap(function _callee4$(_context4) {
			while (1) {
				switch (_context4.prev = _context4.next) {
					case 0:
						return _context4.abrupt('return', _fsExtra2.default.readFileSync(path, 'utf8'));

					case 1:
					case 'end':
						return _context4.stop();
				}
			}
		}, _callee4, undefined);
	}));

	return function getDataFromFile(_x10) {
		return _ref9.apply(this, arguments);
	};
}();

var getData = exports.getData = function () {
	var _ref10 = _asyncToGenerator(regeneratorRuntime.mark(function _callee5(arg, source, data) {
		var _splitSource2, sourceType, path;

		return regeneratorRuntime.wrap(function _callee5$(_context5) {
			while (1) {
				switch (_context5.prev = _context5.next) {
					case 0:
						if (!arg) {
							_context5.next = 2;
							break;
						}

						return _context5.abrupt('return', arg);

					case 2:
						if (!(typeof data === 'string')) {
							_context5.next = 4;
							break;
						}

						return _context5.abrupt('return', data);

					case 4:
						if (source) {
							_context5.next = 6;
							break;
						}

						throw new Error(ERROR_DATA_MISSING);

					case 6:
						_splitSource2 = splitSource(source), sourceType = _splitSource2.sourceType, path = _splitSource2.sourceIdentifier;

						if (!(sourceType !== 'file')) {
							_context5.next = 9;
							break;
						}

						throw new Error(ERROR_DATA_SOURCE_TYPE_UNKNOWN);

					case 9:
						return _context5.abrupt('return', getDataFromFile(path).catch(function (error) {
							var message = error.message;

							if (message.match(/ENOENT/)) {
								throw new Error(getFileDoesNotExistError(path));
							}
							if (message.match(/EACCES/)) {
								throw new Error(getFileUnreadableError(path));
							}
							throw error;
						}));

					case 10:
					case 'end':
						return _context5.stop();
				}
			}
		}, _callee5, undefined);
	}));

	return function getData(_x11, _x12, _x13) {
		return _ref10.apply(this, arguments);
	};
}();